import { Slider, Box } from '@mui/material'
import {
  IconSliderProps,
  LabelPosition,
  SliderOrientation,
  HorizontalModeValuePosition,
} from './IconSlider.types'
import React from 'react'
import { sliderStyle, styles } from './IconSlider.style'

const IconSlider = ({
  valueInterval = { min: 0, max: 100 },
  stepSize = 1,
  label,
  orientation,
  icon,
  value,
  sliderColor,
  size,
  valueDisplay,
  onChange,
}: IconSliderProps) => {
  const isVertical = orientation === SliderOrientation.vertical
  const isLabelOnTop = label?.position === LabelPosition.top
  const isLabelOnBottom = label?.position === LabelPosition.bottom
  const isValueWithLabel =
    valueDisplay?.horizontalModePos === HorizontalModeValuePosition.label
  const getLabelBox = <Box sx={styles.label}> {label?.value} </Box>
  const getValueBox = (
    <Box sx={{ ...styles.value, width: size?.valueWidth ?? 'auto' }}>
      {valueDisplay?.valueFormatter(value)}{' '}
    </Box>
  )

  const getVerticalLabelValueBox = isValueWithLabel ? (
    <Box
      sx={{
        display: 'flex',
        justifyContent: 'space-between',
        width: '100%',
      }}
    >
      {getLabelBox}
      {getValueBox}
    </Box>
  ) : (
    getLabelBox
  )

  const displayValue = valueDisplay !== undefined

  // build slider style from default style by overriding with runtime defined values
  const buildSliderStyle = () => {
    if (sliderColor !== undefined) {
      sliderStyle['& .MuiSlider-rail'].background =
        sliderColor.railColor ?? sliderStyle['& .MuiSlider-rail'].background
      sliderStyle['& .MuiSlider-track'].background =
        sliderColor.trackColor ?? sliderStyle['& .MuiSlider-track'].background
      sliderStyle['& .MuiSlider-thumb'].backgroundColor =
        sliderColor.thumbColor ??
        sliderStyle['& .MuiSlider-thumb'].backgroundColor
    }
    if (size?.track !== undefined) {
      sliderStyle['& .MuiSlider-track'].width = size.track
      sliderStyle['& .MuiSlider-track'].height = size.track
      sliderStyle['& .MuiSlider-mark'].width = size.track
      sliderStyle['& .MuiSlider-mark'].height = size.track
    }

    if (size?.thumb !== undefined) {
      sliderStyle['& .MuiSlider-thumb'].height = size.thumb
      sliderStyle['& .MuiSlider-thumb'].width = size.thumb
    }

    sliderStyle['& .MuiSlider-rail'].width = isVertical
      ? sliderStyle['& .MuiSlider-track'].width
      : 1
    sliderStyle['& .MuiSlider-rail'].height = !isVertical
      ? sliderStyle['& .MuiSlider-track'].height
      : 1

    return sliderStyle
  }

  // ensure value has changed so that onchange does not fire unless the slider moves
  const hasValueChanged = (newValue: any) =>
    Array.isArray(value)
      ? value.some((val, ind) => val !== newValue[ind])
      : newValue !== value

  return (
    <Box
      sx={{
        ...styles.container,
        height: isVertical ? '100%' : 'auto',
      }}
    >
      {isVertical && isLabelOnTop && getLabelBox}

      <Box
        sx={{
          ...styles.container,
          flexDirection: isVertical ? 'column' : 'row',
          height: isVertical ? '100%' : 'auto',
          width: isVertical ? 'auto' : '100%',
        }}
      >
        {displayValue &&
          (isVertical ||
            valueDisplay.horizontalModePos ===
              HorizontalModeValuePosition.end) && (
            <Box
              sx={{
                order: isVertical ? 1 : 3,
                marginLeft: !isVertical ? '13px' : '0px',
                marginBottom: !isVertical ? '15px' : '0px',
              }}
            >
              {getValueBox}
            </Box>
          )}

        <Box
          sx={{
            display: 'flex',
            flexDirection: 'column',
            height: isVertical ? '100%' : 'auto',
            width: isVertical ? 'auto' : '100%',
            marginBottom: isVertical ? '18px' : '0px',
            marginLeft: isVertical ? '0px' : '18px',
            marginTop: isVertical ? '18px' : '0px',
            marginRight: isVertical ? '0px' : '15px',
            order: 2,
            alignItems: 'center',
          }}
        >
          {!isVertical && isLabelOnTop && getVerticalLabelValueBox}
          <Slider
            sx={buildSliderStyle()}
            orientation={orientation}
            value={value}
            step={stepSize}
            getAriaLabel={() => label?.value ?? ''}
            onChange={(event, newValue) =>
              hasValueChanged(newValue) ? onChange(event, newValue) : {}
            }
            min={valueInterval.min}
            max={valueInterval.max}
          />
          {!isVertical && isLabelOnBottom && getVerticalLabelValueBox}
        </Box>
        {icon !== undefined && (
          <Box sx={{ order: isVertical ? 3 : 1 }}>{icon}</Box>
        )}
      </Box>
      {isVertical && isLabelOnBottom && getLabelBox}
    </Box>
  )
}

export default IconSlider
