import React, { useEffect } from 'react'
import { MapIcon, MapProps } from './Map.types'
import { default as OlMap } from 'ol/Map'
import { Tile } from 'ol/layer'
import { OSM } from 'ol/source'
import View from 'ol/View'
import { defaults as controlDefaults } from 'ol/control'
import { Fill, Icon, Stroke, Style } from 'ol/style'
import Feature from 'ol/Feature'
import Point from 'ol/geom/Point'
import VectorSource from 'ol/source/Vector'
import VectorLayer from 'ol/layer/Vector'
import LayerGroup from 'ol/layer/Group'
import BaseLayer from 'ol/layer/Base'
import GeoJSON from 'ol/format/GeoJSON'
import './ol.css'

const Map = ({
  id,
  height,
  width,
  center,
  zoom,
  icons,
  controls,
  tileUrl,
  highlightArea,
}: MapProps) => {
  const createIconStyle = (icon: MapIcon) => {
    return new Style({
      image: new Icon({
        // if any of the following are undefined, the default values defined by ol are used
        anchorXUnits: icon.anchorXUnits,
        anchorYUnits: icon.anchorYUnits,
        src: icon.src,
        anchor: icon.anchor,
        scale: icon.scale,

        //TODO:
        //imgSize
      }),
    })
  }

  useEffect(() => {
    const layers: LayerGroup | BaseLayer[] | LayerGroup = [
      new Tile({
        source: tileUrl ? new OSM({ url: tileUrl }) : new OSM(),
      }),
    ]

    if (highlightArea !== undefined) {
      const geoVectorSource = new VectorSource({
        features: new GeoJSON().readFeatures(highlightArea.geoJson, {
          dataProjection: 'EPSG:4326',
          featureProjection: 'EPSG:3857',
        }),
      })
      layers.push(
        new VectorLayer({
          source: geoVectorSource,
          style: new Style({
            stroke: new Stroke({
              color: highlightArea.strokeColor,
              width: highlightArea.strokeWidth,
            }),
            fill: new Fill({
              color: highlightArea.fillColor,
            }),
          }),
        })
      )
    }

    if (icons !== undefined) {
      const iconFeatures: Feature<Point>[] = []
      icons?.forEach(icon => {
        const feature = new Feature({
          geometry: new Point(icon.position),
          name: icon.name,
        })
        feature.setStyle(createIconStyle(icon))
        iconFeatures.push(feature)
      })
      const vectorSource = new VectorSource({
        features: iconFeatures,
      })
      layers.push(
        new VectorLayer({
          source: vectorSource,
        })
      )
    }

    const map = new OlMap({
      controls: controlDefaults({
        attribution: false,
        zoom: controls?.showZoomButtons,
      }),
      layers,
      view: new View({
        center,
        zoom,
      }),
    })
    map.setTarget('map-' + id)
  }, [center, zoom, icons])

  return <div id={'map-' + id} style={{ height, width }}></div>
}

export default Map
