import React, { useMemo } from 'react'
import {
  Area,
  XAxis,
  YAxis,
  Tooltip,
  ResponsiveContainer,
  Legend,
  Line,
  ComposedChart,
  LegendType,
} from 'recharts'
import { MultiAreaChartProps } from './MultiAreaChart.types'

const MultiAreaChart = ({
  data,
  legend,
  tooltip = { trigger: 'hover' },
  legendDisplayZeroValues = true,
  margin,
}: MultiAreaChartProps) => {
  const mappedData = useMemo(() => {
    let mappedData: any = []
    for (let i = 0; i < data.xAxis.values.length; i++) {
      let dataObj: any = {
        name: data.xAxis.values[i],
      }
      data.areas.forEach((a: any) => (dataObj[a.key] = a.values[i]))
      data.lines?.forEach((l: any) => (dataObj[l.key] = l.values[i]))
      mappedData.push(dataObj)
    }
    return mappedData
  }, [data.areas, data.lines, data.xAxis.values])

  const getLegendType = (values: (number | undefined)[]): LegendType =>
    values.reduce(
      (prev: number | undefined, curr: number | undefined) =>
        (prev !== undefined ? prev : 0) + (curr !== undefined ? curr : 0),
      0
    ) === 0 && legendDisplayZeroValues === false
      ? 'none'
      : 'circle'

  return (
    <>
      <ResponsiveContainer>
        <ComposedChart data={mappedData} margin={margin}>
          <XAxis dataKey="name" fontSize={data.xAxis.fontSize} />
          <YAxis
            domain={
              data.yAxis?.min !== undefined && data.yAxis?.max !== undefined
                ? [data.yAxis.min, data.yAxis.max]
                : undefined
            }
            fontSize={data.yAxis?.fontSize}
            label={data.yAxis?.label}
          />

          {legend.show && (
            <Legend
              verticalAlign="top"
              height={legend.height}
              fontSize={legend.fontSize}
            />
          )}
          {data.areas.map((a: any, i: number) => (
            <Area
              key={'a' + i}
              legendType={getLegendType(a.values)}
              type="monotone"
              name={a.label}
              stackId={a.stackIndex}
              dataKey={a.key}
              fill={a.fill}
              stroke={a.stroke}
              fillOpacity={a.opacity}
              dot={a.dot === undefined ? false : a.dot}
            />
          ))}
          {data.lines?.map((l: any, i: number) => (
            <Line
              key={'l' + i}
              legendType={'circle'}
              type="monotone"
              name={l.label}
              dataKey={l.key}
              stroke={l.stroke}
              opacity={l.opacity}
              strokeWidth={l.width}
              dot={l.dot === undefined ? false : l.dot}
            />
          ))}
          {tooltip.active && (
            <>
              {tooltip.customTooltip !== undefined ? (
                <Tooltip
                  trigger={tooltip.trigger}
                  content={tooltip.customTooltip}
                />
              ) : (
                <Tooltip trigger={tooltip.trigger} />
              )}
            </>
          )}
        </ComposedChart>
      </ResponsiveContainer>
    </>
  )
}

export default MultiAreaChart
