import React, { useEffect, useState } from 'react'
import Slider from '../Slider'
import { styles } from './SlidingDateSelector.style'
import {
  SelectedDate,
  SlidingDateSelectorProps,
} from './SlidingDateSelector.types'
import Utils from './Utils'

export enum Mode {
  DAY,
  MONTH,
  YEAR,
}

interface InitState {
  mode: Mode
  date: SelectedDate
}

const SlidingDateSelector = ({
  labelColor,
  maxYear,
  minYear,
  onDateUpdated,
  primaryButtonColor = '#00a989',
  primaryButtonSelectedColor = '#0092d6',
  secondaryButtonColor = '#ab3e8f',
  initialDate,
  disableTodaySelection = false,
  compact = false,
}: SlidingDateSelectorProps) => {
  const buttonFontSize = '16px'
  const buttonFontSizeCompact = '14px'

  const getInitState = (): InitState => {
    const today = new Date()
    if (initialDate === undefined) {
      return {
        mode: Mode.DAY,
        date: {
          day: today.getDate(),
          month: today.getMonth() + 1,
          year: today.getFullYear(),
        },
      }
    } else {
      let mode = Mode.YEAR
      if (initialDate.month !== undefined) {
        mode = Mode.MONTH
      }
      if (initialDate.day !== undefined) {
        mode = Mode.DAY
      }
      return {
        mode,
        date: initialDate,
      }
    }
  }
  const initState = getInitState()

  const [selectedMode, selectMode] = useState<Mode>(initState.mode)
  const [selectedYear, selectYear] = useState(initState.date.year)
  const [selectedMonth, selectMonth] = useState(initState.date.month)
  const [selectedDay, selectDay] = useState(initState.date.day)

  const setToday = () => {
    const today = new Date()
    const year = today.getFullYear()
    const month = today.getMonth() + 1
    const day = today.getDate()
    selectMode(Mode.DAY)
    selectYear(year)
    selectMonth(month)
    selectDay(day)
  }

  useEffect(() => {
    onDateUpdated({
      day: selectedMode === Mode.DAY ? selectedDay : undefined,
      month:
        selectedMode === Mode.DAY || selectedMode === Mode.MONTH
          ? selectedMonth
          : undefined,
      year: selectedYear,
    })
  }, [selectedMode, selectedDay, selectedMonth, selectedYear, onDateUpdated])

  const updateDay = (newMonth: number | undefined, newYear: number) => {
    if (newMonth !== undefined && selectedDay !== undefined) {
      const newNumberOfDaysInMonth = Utils.calcNumberOfDaysInMonth(
        newYear,
        newMonth
      )
      if (selectedDay > newNumberOfDaysInMonth) {
        selectDay(newNumberOfDaysInMonth)
      }
    }
  }

  const changeMode = (mode: Mode) => selectMode(mode)
  const changeDay = (day: number) => selectDay(day)
  const changeMonth = (month: number) => {
    updateDay(month, selectedYear)
    selectMonth(month)
  }
  const changeYear = (year: number) => {
    updateDay(selectedMonth, year)
    selectYear(year)
  }

  const getMonthName = (monthIn: number) => {
    let objDate = new Date()
    objDate.setDate(1)
    objDate.setMonth(monthIn - 1)

    let locale = 'da-dk'
    let month = objDate.toLocaleString(locale, { month: 'long' })
    return month.charAt(0).toUpperCase() + month.slice(1)
  }

  return (
    <>
      <div style={styles.container}>
        <div style={styles.buttonContainer}>
          <button
            style={{
              ...styles.button,
              fontSize: compact ? buttonFontSizeCompact : buttonFontSize,
              backgroundColor:
                selectedMode === Mode.DAY
                  ? primaryButtonSelectedColor
                  : primaryButtonColor,
            }}
            onClick={() => {
              changeMode(Mode.DAY)
            }}
          >
            Et døgn
          </button>

          <button
            style={{
              ...styles.button,
              fontSize: compact ? buttonFontSizeCompact : buttonFontSize,
              backgroundColor:
                selectedMode === Mode.MONTH
                  ? primaryButtonSelectedColor
                  : primaryButtonColor,
            }}
            onClick={() => {
              changeMode(Mode.MONTH)
            }}
          >
            En måned
          </button>

          <button
            style={{
              ...styles.button,
              fontSize: compact ? buttonFontSizeCompact : buttonFontSize,
              backgroundColor:
                selectedMode === Mode.YEAR
                  ? primaryButtonSelectedColor
                  : primaryButtonColor,
            }}
            onClick={() => {
              changeMode(Mode.YEAR)
            }}
          >
            Et år
          </button>
        </div>
        {!disableTodaySelection && (
          <div style={styles.secondaryButtonContainer}>
            <button
              style={{
                ...styles.button,
                fontSize: compact ? buttonFontSizeCompact : buttonFontSize,
                backgroundColor: secondaryButtonColor,
              }}
              onClick={() => setToday()}
            >
              I dag
            </button>
          </div>
        )}
      </div>
      <Slider
        compact={compact}
        labelColor={labelColor}
        label=""
        valueAsString={selectedYear ? selectedYear.toString() : ''}
        value={selectedYear}
        minValue={minYear}
        maxValue={maxYear}
        changeValue={changeYear}
      />
      {selectedMonth !== undefined && (
        <Slider
          compact={compact}
          hidden={selectedMode !== Mode.MONTH && selectedMode !== Mode.DAY}
          labelColor={labelColor}
          label=""
          valueAsString={getMonthName(selectedMonth)}
          value={selectedMonth}
          minValue={1}
          maxValue={12}
          changeValue={changeMonth}
        />
      )}
      {selectedDay !== undefined && selectedMonth !== undefined && (
        <Slider
          compact={compact}
          hidden={selectedMode !== Mode.DAY}
          labelColor={labelColor}
          label=""
          valueAsString={selectedDay ? selectedDay.toString() + '.' : ''}
          value={selectedDay}
          minValue={1}
          maxValue={Utils.calcNumberOfDaysInMonth(selectedYear, selectedMonth)}
          changeValue={changeDay}
        />
      )}
    </>
  )
}

export default SlidingDateSelector
